/*****************************************************************
 
  treeprivate.h
 
  Copyright (c) 2003-2004 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de
 
  http://gmerlin.sourceforge.net
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 
*****************************************************************/

/* Flags should not be changed */

#define BG_ALBUM_EXPANDED    (1<<1)
#define BG_ALBUM_ERROR       (1<<2)
#define BG_ALBUM_CAN_EJECT   (1<<3)

typedef struct bg_shuffle_list_s
  {
  bg_album_entry_t * entry;
  bg_album_t       * album;
  struct bg_shuffle_list_s * next;
  struct bg_shuffle_list_s * prev;
  } bg_shuffle_list_t;

void bg_shuffle_list_destroy(bg_shuffle_list_t*l);

/* Types of the album */

typedef struct
  {
  bg_plugin_registry_t * plugin_reg;
  char * filename_template;
  char * directory;

  /* Highest Track ID */
  
  int highest_id;

  bg_album_t       * current_album;
  bg_album_entry_t * current_entry;

  void (*set_current_callback)(void * data, bg_album_t * current_album,
                               const bg_album_entry_t * current_entry);
  void * set_current_callback_data;

  void (*play_callback)(void * data);
  void * play_callback_data;

  int (*userpass_callback)(const char * resource, char ** user, char ** pass, int * save_password,
                           void * data);
  void * userpass_callback_data;
  
  
  bg_plugin_handle_t * load_handle;

  /* Configuration stuff */

  int use_metadata;
  char * metadata_format;

  /* Shuffle list */

  bg_shuffle_list_t * shuffle_list;

  /* Favourites album */

  bg_album_t       * favourites;
  bg_input_callbacks_t input_callbacks;

  /* Stored authentication data */
  char * username;
  char * password;
  int save_auth;
  
  } bg_album_common_t;

void bg_album_common_prepare_callbacks(bg_album_common_t*,bg_album_entry_t * entry);

void bg_album_common_set_auth_info(bg_album_common_t*, bg_album_entry_t*);

struct bg_album_s
  {
  bg_album_type_t type;
  bg_album_common_t * com;

  int open_count;
  
  char * name;        /* Name for dialog boxes      */
  char * location;    /* Album filename or device   */

  char * disc_name;   /* Set by open_removable */

  
  const bg_plugin_info_t * plugin_info;
  bg_plugin_handle_t      * handle;
  
  int    flags;

  struct bg_album_s       * children;
  struct bg_album_s       * next;
  struct bg_album_s       * parent;
  
  bg_album_entry_t * entries;
  
  void (*change_callback)(bg_album_t * a, void * data);
  void * change_callback_data;

  void (*name_change_callback)(bg_album_t * a, const char * name, void * data);
  void * name_change_callback_data;
  
  /* Coordinates in the screen */

  //  int x;
  //  int y;
  //  int width;
  //  int height;
  
  /* Path for loading files */
    
  //  char * open_path;

  /* Here, dialogs can store additional config data */
    
  bg_cfg_section_t * cfg_section;
    
  };

/* album.c */

void bg_album_update_entry(bg_album_t * album,
                           bg_album_entry_t * entry,
                           bg_track_info_t  * track_info);

int bg_album_get_unique_id(bg_album_t * album);

bg_album_t * bg_album_create(bg_album_common_t * com, bg_album_type_t type,
                             bg_album_t * parent);


void bg_album_set_default_location(bg_album_t * album);

void bg_album_destroy(bg_album_t * album);

void bg_album_load(bg_album_t * album, const char * filename);

/* Notify the album of changes */

void bg_album_changed(bg_album_t * album);

void bg_album_insert_entries_after(bg_album_t * album,
                                   bg_album_entry_t * new_entries,
                                   bg_album_entry_t * before);

void bg_album_insert_entries_before(bg_album_t * album,
                                    bg_album_entry_t * new_entries,
                                    bg_album_entry_t * after);

int bg_album_refresh_entry(bg_album_t *,
                           bg_album_entry_t * entry);

/*
 *   Load a single URL, perform redirection and return
 *   zero (for error) or more entries
 */

bg_album_entry_t * bg_album_load_url(bg_album_t * album,
                                     char * url,
                                     const char * plugin_long_name);

struct bg_mediatree_s
  {
  bg_album_common_t com;

  char * filename;
  
  bg_album_t       * children;

  /* The following ones must always be there */

  bg_album_t       * incoming;
  
  bg_plugin_handle_t * load_handle;
  
  void (*change_callback)(bg_media_tree_t*, void*);
  void * change_callback_data;
    
  void (*error_callback)(bg_media_tree_t*, void*, const char*);
  void * error_callback_data;
  
  /* Current entry in shuffle list */

  bg_shuffle_list_t * shuffle_current;
  bg_shuffle_mode_t last_shuffle_mode;

  int purge_directory;

  /* Here, dialogs can store additional config data */
    
  bg_cfg_section_t * cfg_section;
  };

/* Purge the directory (remove unused albums) */

void bg_media_tree_purge_directory(bg_media_tree_t * t);

/* Create a unique filename */

// char * bg_media_tree_new_album_filename(bg_media_tree_t * tree);

void bg_album_entry_destroy(bg_album_entry_t * entry);

bg_album_entry_t * bg_album_entry_create(bg_album_t * album);

